@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # The internetarchive module is a Python/CLI interface to Archive.org.
 #
@@ -28,3 +27,12 @@ internetarchive.exceptions
 
 class AuthenticationError(Exception):
     """Authentication Failed"""
+
+
+class ItemLocateError(Exception):
+    def __init__(self, *args, **kwargs):
+        default_message = 'Item cannot be located because it is dark or does not exist.'
+        if args or kwargs:
+            super().__init__(*args, **kwargs)
+        else:
+            super().__init__(default_message)
